#ifndef GCSA_H
#define GCSA_H

#include <fstream>
#include <vector>

#include <bits/deltavector.h>
#include <bits/rlevector.h>

#include "graph.h"


namespace CSA
{

  
class GCSA
{
  public:

//--------------------------------------------------------------------------
//  CONSTRUCTION
//--------------------------------------------------------------------------

    const static usint ARRAY_BLOCK_SIZE = 32;
    const static usint CHILDREN_BLOCK_SIZE = 32;
    const static usint OUTGOING_BLOCK_SIZE = 32;
    const static usint SAMPLE_BLOCK_SIZE = 32;
    const static usint SAMPLE_RATE = 16;

    const static usint ALPHABET_SIZE = 5;
    const static std::string ALPHABET;

    GCSA(const std::string& base_name);
    GCSA(PathGraph& graph, bool print = false);
    ~GCSA();

    void writeTo(const std::string& base_name) const;

    bool isOk() const;

//--------------------------------------------------------------------------
//  REPORTING
//--------------------------------------------------------------------------

    // Returns the size of the data structure.
    usint reportSize(bool print = false) const;

    usint getSize() const;

//--------------------------------------------------------------------------
//  SEARCHING
//--------------------------------------------------------------------------

    // Returns the SA range for suffixes starting with the pattern.
    pair_type find(const std::string& pattern) const;


    // Returns the set of unique node values for the given BWT range of nodes.
    // User must delete the returned vector.
    std::vector<usint>* locate(pair_type range) const;
    std::vector<usint>* locate(std::vector<pair_type>& ranges) const;

//--------------------------------------------------------------------------
//  SUPPORT FOR APPROXIMATE MATCHING MODULES
//--------------------------------------------------------------------------

    pair_type getSARange() const;
    pair_type getBWTRange() const;
    pair_type getCharRange(usint c) const;

    void convertToBWTRange(pair_type& sa_range) const;
    void convertToSARange(pair_type& bwt_range) const;
    void convertToSARange(std::vector<pair_type>& bwt_ranges) const;

    // This is an unsafe function that does not check its parameters.
    pair_type LF(pair_type bwt_range, usint c) const;

    // User must free the returned vector.
    std::vector<usint>* locateRange(pair_type range) const;
    std::vector<usint>* locateRanges(std::vector<pair_type>& ranges) const;

//--------------------------------------------------------------------------
//  INTERNAL VARIABLES
//--------------------------------------------------------------------------

  private:
    bool ok;

    DeltaVector* array[ALPHABET_SIZE];    // BWT
    RLEVector* first_children;            // F
    RLEVector* outgoing;                  // M

    DeltaVector* sampled_positions;
    ReadBuffer* samples;

    usint counts[ALPHABET_SIZE];
    usint cumulative[ALPHABET_SIZE + 1];  // C

//--------------------------------------------------------------------------
//  INTERNAL STUFF
//--------------------------------------------------------------------------

    bool exists(usint c) const;
    usint getCharacter(usint index) const;  // Parameter should be a rank of an outgoing edge.
    pair_type convertToNodeRange(pair_type edge_range) const;
    pair_type Psi(pair_type range) const; // Follows the first outgoing edge.

    void locate(pair_type range, std::vector<usint>& vec) const;

    void fillArrayC();

    // These are not allowed.
    GCSA();
    GCSA(const GCSA&);
    GCSA& operator = (const GCSA&);
};


} // namespace CSA


#endif // GCSA_H
