#include <cstdlib>
#include <fstream>
#include <iostream>

#include <misc/definitions.h>


using namespace CSA;


std::string gapChars = " =-_";

int
main(int argc, char** argv)
{
  std::cout << "Alignment transposer" << std::endl;
  std::cout << std::endl;
  if(argc < 3)
  {
    std::cout << "Usage: transpose alignment_file output_file [endmarker]" << std::endl;
    return 1;
  }

  std::cout << "Alignment file: " << argv[1] << std::endl;
  std::ifstream alignment_file(argv[1], std::ios_base::binary);
  if(!alignment_file)
  {
    std::cerr << "Error opening alignment file!" << std::endl;
    return 2;
  }

  std::cout << "Output file: " << argv[2] << std::endl;
  std::ofstream output_file(argv[2], std::ios_base::binary);
  if(!output_file)
  {
    std::cerr << "Error creating output file!" << std::endl;
    return 3;
  }

  usint endmarker = '\n';
  if(argc >= 4) { endmarker = atoi(argv[3]); }
  std::cout << "End marker: " << endmarker << std::endl;

  // Determine the number of sequences.
  std::string line;
  std::getline(alignment_file, line);
  usint sequences = line.size();
  std::cout << "Sequences: " << sequences << std::endl;
  std::cout << std::endl;

  usint counts[CHARS];
  for(usint c = 0; c < CHARS; c++) { counts[c] = 0; }

  for(usint i = 0; i < sequences; i++)
  {
    alignment_file.clear();
    alignment_file.seekg(0, std::ios_base::beg);
    usint len = 0;
    while(!alignment_file.eof())
    {
      getline(alignment_file, line);
      if(line.empty()) { continue; }
      if(gapChars.find(line[i]) == std::string::npos)
      {
        output_file.put(line[i]); len++;
        counts[(int)line[i]]++;
      }
    }
    output_file.put(endmarker);

    std::cout << "Sequence " << i << ", length " << len << std::endl;
  }
  std::cout << std::endl;

  std::cout << "Character counts: " << std::endl;
  for(usint c = 0; c < CHARS; c++)
  {
    if(counts[c] == 0) { continue; }
    std::cout << "  " << ((char)c) << " (" << c << "): " << counts[c] << std::endl;
  }
  std::cout << std::endl;

  return 0;
}
